@@ -1,17 +1,34 @@
+/*   This file is part of Motion.
+ *
+ *   Motion is free software: you can redistribute it and/or modify
+ *   it under the terms of the GNU General Public License as published by
+ *   the Free Software Foundation, either version 2 of the License, or
+ *   (at your option) any later version.
+ *
+ *   Motion is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU General Public License for more details.
+ *
+ *   You should have received a copy of the GNU General Public License
+ *   along with Motion.  If not, see <https://www.gnu.org/licenses/>.
+ */
+
 /*    alg.c
  *
  *    Detect changes in a video stream.
  *    Copyright 2001 by Jeroen Vreeken (pe1rxq@amsat.org)
- *    This software is distributed under the GNU public license version 2
- *    See also the file 'COPYING'.
  *
  */
+
 #include "motion.h"
+#include "util.h"
+#include "draw.h"
 #include "alg.h"
 
 #ifdef __MMX__
-#define HAVE_MMX
-#include "mmx.h"
+    #define HAVE_MMX
+    #include "mmx.h"
 #endif
 
 #define MAX2(x, y) ((x) > (y) ? (x) : (y))
@@ -78,16 +95,16 @@ void alg_locate_center_size(struct images *imgs, int width, int height, struct c
         for (y = 0; y < height; y++) {
             for (x = 0; x < width; x++) {
                 if (*(labels++) & 32768) {
-                    if (x > cent->x)
+                    if (x > cent->x) {
                         xdist += x - cent->x;
-                    else if (x < cent->x)
+                    } else if (x < cent->x) {
                         xdist += cent->x - x;
-
-                    if (y > cent->y)
+                    }
+                    if (y > cent->y) {
                         ydist += y - cent->y;
-                    else if (y < cent->y)
+                    } else if (y < cent->y) {
                         ydist += cent->y - y;
-
+                    }
                     centc++;
                 }
             }
@@ -97,16 +114,16 @@ void alg_locate_center_size(struct images *imgs, int width, int height, struct c
         for (y = 0; y < height; y++) {
             for (x = 0; x < width; x++) {
                 if (*(out++)) {
-                    if (x > cent->x)
+                    if (x > cent->x) {
                         xdist += x - cent->x;
-                    else if (x < cent->x)
+                    } else if (x < cent->x) {
                         xdist += cent->x - x;
-
-                    if (y > cent->y)
+                    }
+                    if (y > cent->y) {
                         ydist += y - cent->y;
-                    else if (y < cent->y)
+                    } else if (y < cent->y) {
                         ydist += cent->y - y;
-
+                    }
                     centc++;
                 }
             }
@@ -126,25 +143,28 @@ void alg_locate_center_size(struct images *imgs, int width, int height, struct c
         cent->maxy = cent->y + ydist / centc * 2;
     }
 
-    if (cent->maxx > width - 1)
+    if (cent->maxx > width - 1) {
         cent->maxx = width - 1;
-    else if (cent->maxx < 0)
+    } else if (cent->maxx < 0) {
         cent->maxx = 0;
-
-    if (cent->maxy > height - 1)
+    }
+    if (cent->maxy > height - 1) {
         cent->maxy = height - 1;
-    else if (cent->maxy < 0)
+    } else if (cent->maxy < 0) {
         cent->maxy = 0;
+    }
 
-    if (cent->minx > width - 1)
+    if (cent->minx > width - 1) {
         cent->minx = width - 1;
-    else if (cent->minx < 0)
+    } else if (cent->minx < 0) {
         cent->minx = 0;
+    }
 
-    if (cent->miny > height - 1)
+    if (cent->miny > height - 1) {
         cent->miny = height - 1;
-    else if (cent->miny < 0)
+    } else if (cent->miny < 0) {
         cent->miny = 0;
+    }
 
     /* Align for better locate box handling */
     cent->minx += cent->minx % 2;
@@ -370,8 +390,9 @@ void alg_noise_tune(struct context *cnt, unsigned char *new)
     for (; i > 0; i--) {
         diff = ABS(*ref - *new);
 
-        if (mask)
+        if (mask) {
             diff = ((diff * *mask++) / 255);
+        }
 
         if (*smartmask) {
             sum += diff + 1;
@@ -383,11 +404,13 @@ void alg_noise_tune(struct context *cnt, unsigned char *new)
         smartmask++;
     }
 
-    if (count > 3)  /* Avoid divide by zero. */
+    if (count > 3) {
+        /* Avoid divide by zero. */
         sum /= count / 3;
+    }
 
     /* 5: safe, 4: regular, 3: more sensitive */
-    cnt->noise = 4 + (cnt->noise + sum) / 2;
+    cnt->noise = 4 + (cnt->conf.noise_level / 2) + (cnt->noise + sum) / 2;
 }
 
 /**
@@ -399,22 +422,26 @@ void alg_threshold_tune(struct context *cnt, int diffs, int motion)
     int i;
     int sum = 0, top = diffs;
 
-    if (!diffs)
+    if (!diffs) {
         return;
+    }
 
-    if (motion)
+    if (motion) {
         diffs = cnt->threshold / 4;
+    }
 
     for (i = 0; i < THRESHOLD_TUNE_LENGTH - 1; i++) {
         sum += cnt->diffs_last[i];
 
-        if (cnt->diffs_last[i + 1] && !motion)
+        if (cnt->diffs_last[i + 1] && !motion) {
             cnt->diffs_last[i] = cnt->diffs_last[i + 1];
-        else
+        } else {
             cnt->diffs_last[i] = cnt->threshold / 4;
+        }
 
-        if (cnt->diffs_last[i] > top)
+        if (cnt->diffs_last[i] > top) {
             top = cnt->diffs_last[i];
+        }
     }
 
     sum += cnt->diffs_last[i];
@@ -422,11 +449,13 @@ void alg_threshold_tune(struct context *cnt, int diffs, int motion)
 
     sum /= THRESHOLD_TUNE_LENGTH / 4;
 
-    if (sum < top * 2)
+    if (sum < top * 2) {
         sum = top * 2;
+    }
 
-    if (sum < cnt->conf.threshold)
+    if (sum < cnt->conf.threshold) {
         cnt->threshold = (cnt->threshold + sum) / 2;
+    }
 }
 
 /*
@@ -463,8 +492,9 @@ static int iflood(int x, int y, int width, int height,
     Segment stack[MAXS], *sp = stack;    /* Stack of filled segments. */
     int count = 0;
 
-    if (x < 0 || x >= width || y < 0 || y >= height)
+    if (x < 0 || x >= width || y < 0 || y >= height) {
         return 0;
+    }
 
     PUSH(y, x, x, 1);             /* Needed in some cases. */
     PUSH(y+1, x, x, -1);          /* Seed segment (popped 1st). */
@@ -481,13 +511,15 @@ static int iflood(int x, int y, int width, int height,
             count++;
         }
 
-        if (x >= x1)
+        if (x >= x1) {
             goto skip;
+        }
 
         l = x + 1;
 
-        if (l < x1)
+        if (l < x1) {
             PUSH(y, l, x1 - 1, -dy);  /* Leak on left? */
+        }
 
         x = x1 + 1;
 
@@ -499,12 +531,13 @@ static int iflood(int x, int y, int width, int height,
 
             PUSH(y, l, x - 1, dy);
 
-            if (x > x2 + 1)
+            if (x > x2 + 1) {
                 PUSH(y, x2 + 1, x - 1, -dy);  /* Leak on right? */
+            }
 
             skip:
 
-            for (x++; x <= x2 && !(out[y * width + x] != 0 && labels[y * width + x] == oldvalue); x++);
+            for (x++; x <= x2 && !(out[y * width + x] != 0 && labels[y * width + x] == oldvalue); x++) {}
 
             l = x;
         } while (x <= x2);
@@ -548,8 +581,9 @@ static int alg_labeling(struct context *cnt)
             }
 
             /* Already visited by iflood */
-            if (labels[pixelpos] > 0)
+            if (labels[pixelpos] > 0) {
                 continue;
+            }
 
             labelsize = iflood(ix, iy, width, height, out, labels, current_label, 0);
 
@@ -563,8 +597,9 @@ static int alg_labeling(struct context *cnt)
                     labelsize = iflood(ix, iy, width, height, out, labels, current_label + 32768, current_label);
                     imgs->labelgroup_max += labelsize;
                     imgs->labels_above++;
-                } else if(max_under < labelsize)
+                } else if(max_under < labelsize) {
                     max_under = labelsize;
+                }
 
                 if (imgs->labelsize_max < labelsize) {
                     imgs->labelsize_max = labelsize;
@@ -627,10 +662,11 @@ static int dilate9(unsigned char *img, int width, int height, void *buffer)
         row3 = rowTemp;
 
         /* If we're at the last row, fill with zeros, otherwise copy from img. */
-        if (y == height - 1)
+        if (y == height - 1) {
             memset(row3, 0, width);
-        else
+        } else {
             memcpy(row3, yp+width, width);
+        }
 
         /* Init slots 0 and 1 in the moving window. */
         window[0] = MAX3(row1[0], row2[0], row3[0]);
@@ -652,10 +688,11 @@ static int dilate9(unsigned char *img, int width, int height, void *buffer)
              * If the value is larger than the current max, use it. Otherwise,
              * calculate a new max (because the new value may not be the max.
              */
-            if (latest >= blob)
+            if (latest >= blob) {
                 blob = latest;
-            else
+            } else {
                 blob = MAX3(window[0], window[1], window[2]);
+            }
 
             /* Write the max value (blob) to the image. */
             if (blob != 0) {
@@ -664,8 +701,9 @@ static int dilate9(unsigned char *img, int width, int height, void *buffer)
             }
 
             /* Wrap around the window index if necessary. */
-            if (++widx == 3)
+            if (++widx == 3) {
                 widx = 0;
+            }
         }
 
         /* Store zeros in the vertical sides. */
@@ -710,10 +748,11 @@ static int dilate5(unsigned char *img, int width, int height, void *buffer)
         row3 = rowTemp;
 
         /* If we're at the last row, fill with zeros, otherwise copy from img. */
-        if (y == height - 1)
+        if (y == height - 1) {
             memset(row3, 0, width);
-        else
+        } else {
             memcpy(row3, yp + width, width);
+        }
 
         /* Init mem and set blob to force an evaluation of the entire + shape. */
         mem = MAX2(row2[0], row2[1]);
@@ -766,10 +805,11 @@ static int erode9(unsigned char *img, int width, int height, void *buffer, unsig
         memcpy(Row1, Row2, width);
         memcpy(Row2, Row3, width);
 
-        if (y == height-1)
+        if (y == height-1) {
             memset(Row3, flag, width);
-        else
+        } else {
             memcpy(Row3, img + (y+1) * width, width);
+        }
 
         for (i = width - 2; i >= 1; i--) {
             if (Row1[i - 1] == 0 ||
@@ -780,10 +820,11 @@ static int erode9(unsigned char *img, int width, int height, void *buffer, unsig
                 Row2[i + 1] == 0 ||
                 Row3[i - 1] == 0 ||
                 Row3[i]     == 0 ||
-                Row3[i + 1] == 0)
+                Row3[i + 1] == 0) {
                 img[y * width + i] = 0;
-            else
+            } else {
                 sum++;
+            }
         }
 
         img[y * width] = img[y * width + width - 1] = flag;
@@ -810,20 +851,22 @@ static int erode5(unsigned char *img, int width, int height, void *buffer, unsig
         memcpy(Row1, Row2, width);
         memcpy(Row2, Row3, width);
 
-        if (y == height-1)
+        if (y == height-1) {
             memset(Row3, flag, width);
-        else
+        } else {
             memcpy(Row3, img + (y + 1) * width, width);
+        }
 
         for (i = width - 2; i >= 1; i--) {
             if (Row1[i]     == 0 ||
                 Row2[i - 1] == 0 ||
                 Row2[i]     == 0 ||
                 Row2[i + 1] == 0 ||
-                Row3[i]     == 0)
+                Row3[i]     == 0) {
                 img[y * width + i] = 0;
-            else
+            } else {
                 sum++;
+            }
         }
 
         img[y * width] = img[y * width + width - 1] = flag;
@@ -847,13 +890,15 @@ int alg_despeckle(struct context *cnt, int olddiffs)
     for (i = 0; i < len; i++) {
         switch (cnt->conf.despeckle_filter[i]) {
         case 'E':
-            if ((diffs = erode9(out, width, height, common_buffer, 0)) == 0)
+            if ((diffs = erode9(out, width, height, common_buffer, 0)) == 0) {
                 i = len;
+            }
             done = 1;
             break;
         case 'e':
-            if ((diffs = erode5(out, width, height, common_buffer, 0)) == 0)
+            if ((diffs = erode5(out, width, height, common_buffer, 0)) == 0) {
                 i = len;
+            }
             done = 1;
             break;
         case 'D':
@@ -875,8 +920,9 @@ int alg_despeckle(struct context *cnt, int olddiffs)
 
     /* If conf.despeckle_filter contains any valid action EeDdl */
     if (done) {
-        if (done != 2)
+        if (done != 2) {
             cnt->imgs.labelsize_max = 0; // Disable Labeling
+        }
         return diffs;
     } else {
         cnt->imgs.labelsize_max = 0; // Disable Labeling
@@ -900,23 +946,26 @@ void alg_tune_smartmask(struct context *cnt)
 
     for (i = 0; i < motionsize; i++) {
         /* Decrease smart_mask sensitivity every 5*speed seconds only. */
-        if (smartmask[i] > 0)
+        if (smartmask[i] > 0) {
             smartmask[i]--;
+        }
         /* Increase smart_mask sensitivity based on the buffered values. */
         diff = smartmask_buffer[i]/sensitivity;
 
         if (diff) {
-            if (smartmask[i] <= diff + 80)
+            if (smartmask[i] <= diff + 80) {
                 smartmask[i] += diff;
-            else
+            } else {
                 smartmask[i] = 80;
+            }
             smartmask_buffer[i] %= sensitivity;
         }
         /* Transfer raw mask to the final stage when above trigger value. */
-        if (smartmask[i] > 20)
+        if (smartmask[i] > 20) {
             smartmask_final[i] = 0;
-        else
+        } else {
             smartmask_final[i] = 255;
+        }
     }
     /* Further expansion (here:erode due to inverted logic!) of the mask. */
     diff = erode9(smartmask_final, cnt->imgs.width, cnt->imgs.height,
@@ -943,10 +992,10 @@ int alg_diff_standard(struct context *cnt, unsigned char *new)
     unsigned char *mask = imgs->mask;
     unsigned char *smartmask_final = imgs->smartmask_final;
     int *smartmask_buffer = imgs->smartmask_buffer;
-#ifdef HAVE_MMX
-    mmx_t mmtemp; /* Used for transferring to/from memory. */
-    int unload;   /* Counter for unloading diff counts. */
-#endif
+    #ifdef HAVE_MMX
+        mmx_t mmtemp; /* Used for transferring to/from memory. */
+        int unload;   /* Counter for unloading diff counts. */
+    #endif
 
     i = imgs->motionsize;
     memset(out + i, 128, i / 2); /* Motion pictures are now b/w i.o. green */
@@ -957,208 +1006,225 @@ int alg_diff_standard(struct context *cnt, unsigned char *new)
      */
     memset(out, 0, i);
 
-#ifdef HAVE_MMX
-    /*
-     * NOTE: The Pentium has two instruction pipes: U and V. I have grouped MMX
-     * instructions in pairs according to how I think they will be scheduled in
-     * the U and V pipes. Due to pairing constraints, the V pipe will sometimes
-     * be empty (for example, memory access always goes into the U pipe).
-     *
-     * The following MMX registers are kept throughout the loop:
-     * mm5 - 8 separate diff counters (unloaded periodically)
-     * mm6 - mask: 00ff 00ff 00ff 00ff
-     * mm7 - noise level as 8 packed bytes
-     *
-     * -- Per Jonsson
-     */
-
-    /*
-     * To avoid a div, we work with differences multiplied by 255 in the
-     * default case and *mask otherwise. Thus, the limit to compare with is
-     * 255 * (noise + 1) - 1).
-     */
-    mmtemp.uw[0] = mmtemp.uw[1] = mmtemp.uw[2] = mmtemp.uw[3] =
-                   (unsigned short)(noise * 255 + 254);
-
-    /*
-     * Reset mm5 to zero, set the mm6 mask, and store the multiplied noise
-     * level as four words in mm7.
-     */
-    movq_m2r(mmtemp, mm7);             /* U */
-    pcmpeqb_r2r(mm6, mm6);             /* V */
-
-    pxor_r2r(mm5, mm5);                /* U */
-    psrlw_i2r(8, mm6);                 /* V */
-
-    /*
-     * We must unload mm5 every 255th round, because the diffs accumulate
-     * in each packed byte, which can hold at most 255 diffs before it
-     * gets saturated.
-     */
-    unload = 255;
-
-    for (; i > 7; i -= 8) {
-        /* Calculate abs(*ref-*new) for 8 pixels in parallel. */
-        movq_m2r(*ref, mm0);           /* U: mm0 = r7 r6 r5 r4 r3 r2 r1 r0 */
-        pxor_r2r(mm4, mm4);            /* V: mm4 = 0 */
-
-        movq_m2r(*new, mm1);           /* U: mm1 = n7 n6 n5 n4 n3 n2 n1 n0 */
-        movq_r2r(mm0, mm2);            /* V: mm2 = r7 r6 r5 r4 r3 r2 r1 r0 */
-
-        /* These subtractions are saturated, i.e. won't go below 0. */
-        psubusb_r2r(mm1, mm0);         /* U: mm0 = (r7-n7) ... (r0-n0) */
-        psubusb_r2r(mm2, mm1);         /* V: mm1 = (n7-r7) ... (n0-r0) */
-
-        /* Each byte dX in mm0 is abs(nX-rX). */
-        por_r2r(mm1, mm0);             /* U: mm0 = d7 d6 d5 d4 d3 d2 d1 d0 */
-
-        /* Expand the absolute differences to words in mm0 and mm1. */
-        movq_r2r(mm0, mm1);            /* U: mm1 = d7 d6 d5 d4 d3 d2 d1 d0 */
-        punpcklbw_r2r(mm4, mm0);       /* V: mm0 =    d3    d2    d1    d0 */
-
-        punpckhbw_r2r(mm4, mm1);       /* U: mm1 =    d7    d6    d5    d4 */
-
-        if (mask) {
-            /*
-             * Load and expand 8 mask bytes to words in mm2 and mm3. Then
-             * multiply by mm0 and mm1, respectively.
-             */
-            movq_m2r(*mask, mm2);      /* U: mm2 = m7 m6 m5 m4 m3 m2 m1 m0 */
-
-            movq_r2r(mm2, mm3);        /* U: mm3 = m7 m6 m5 m4 m3 m2 m1 m0 */
-            punpcklbw_r2r(mm4, mm2);   /* v: mm2 =    m3    m2    m1    m0 */
-
-            punpckhbw_r2r(mm4, mm3);   /* U: mm3 =    m7    m6    m5    m4 */
-            pmullw_r2r(mm2, mm0);      /* V: mm0 = (d3*m3) ... (d0*m0) */
-
-            pmullw_r2r(mm3, mm1);      /* U: mm1 = (d7*m7) ... (d4*m4) */
-
-            mask += 8;
-        } else {
-            /*
-             * Not using mask - multiply the absolute differences by 255. We
-             * do this by left-shifting 8 places and then subtracting dX.
-             */
-            movq_r2r(mm0, mm2);        /* U: mm2 =    d3    d2    d1    d0 */
-            psllw_i2r(8, mm0);         /* V: mm2 = (256*d3) ... (256*d0) */
-
-            movq_r2r(mm1, mm3);        /* U: mm3 =    d7    d6    d5    d4 */
-            psllw_i2r(8, mm1);         /* V: mm3 = (256*d7) ... (256*d4) */
-
-            psubusw_r2r(mm2, mm0);     /* U */
-            psubusw_r2r(mm3, mm1);     /* V */
-        }
+    #ifdef HAVE_MMX
+        /*
+        * NOTE: The Pentium has two instruction pipes: U and V. I have grouped MMX
+        * instructions in pairs according to how I think they will be scheduled in
+        * the U and V pipes. Due to pairing constraints, the V pipe will sometimes
+        * be empty (for example, memory access always goes into the U pipe).
+        *
+        * The following MMX registers are kept throughout the loop:
+        * mm5 - 8 separate diff counters (unloaded periodically)
+        * mm6 - mask: 00ff 00ff 00ff 00ff
+        * mm7 - noise level as 8 packed bytes
+        *
+        * -- Per Jonsson
+        */
 
         /*
-         * Next, compare the multiplied absolute differences with the multiplied
-         * noise level (repeated as 4 words in mm7), resulting in a "motion flag"
-         * for each pixel.
-         *
-         * Since pcmpgtw performs signed comparisons, we have to subtract noise,
-         * test for equality to 0 and then invert the result.
-         *
-         * Note that it is safe to generate the "motion flags" before the
-         * smartmask code, as all that can happen is that individual flags get
-         * reset to 0 because of the smartmask.
-         */
-        psubusw_r2r(mm7, mm0);         /* U: subtract by (multiplied) noise */
-        psubusw_r2r(mm7, mm1);         /* V */
+        * To avoid a div, we work with differences multiplied by 255 in the
+        * default case and *mask otherwise. Thus, the limit to compare with is
+        * 255 * (noise + 1) - 1).
+        */
+        mmtemp.uw[0] = mmtemp.uw[1] = mmtemp.uw[2] = mmtemp.uw[3] =
+                    (unsigned short)(noise * 255 + 254);
 
-        pcmpeqw_r2r(mm4, mm0);         /* U: test for equality with 0 */
-        pcmpeqw_r2r(mm4, mm1);         /* V */
+        /*
+        * Reset mm5 to zero, set the mm6 mask, and store the multiplied noise
+        * level as four words in mm7.
+        */
+        movq_m2r(mmtemp, mm7);             /* U */
+        pcmpeqb_r2r(mm6, mm6);             /* V */
 
-        pand_r2r(mm6, mm0);            /* U: convert 0xffff -> 0x00ff */
-        pand_r2r(mm6, mm1);            /* V */
+        pxor_r2r(mm5, mm5);                /* U */
+        psrlw_i2r(8, mm6);                 /* V */
 
-        pxor_r2r(mm6, mm0);            /* U: invert the result */
-        pxor_r2r(mm6, mm1);            /* V */
+        /*
+        * We must unload mm5 every 255th round, because the diffs accumulate
+        * in each packed byte, which can hold at most 255 diffs before it
+        * gets saturated.
+        */
+        unload = 255;
 
-        /* Each fX is the "motion flag" = 0 for no motion, 0xff for motion. */
-        packuswb_r2r(mm1, mm0);        /* U: mm0 = f7 f6 f5 f4 f3 f2 f1 f0 */
+        for (; i > 7; i -= 8) {
+            /* Calculate abs(*ref-*new) for 8 pixels in parallel. */
+            movq_m2r(*ref, mm0);           /* U: mm0 = r7 r6 r5 r4 r3 r2 r1 r0 */
+            pxor_r2r(mm4, mm4);            /* V: mm4 = 0 */
 
-        if (smartmask_speed) {
-            /*
-             * Apply the smartmask. Basically, if *smartmask_final is 0, the
-             * corresponding "motion flag" in mm0 will be reset.
-             */
-            movq_m2r(*smartmask_final, mm3); /* U: mm3 = s7 s6 s5 s4 s3 s2 s1 s0 */
+            movq_m2r(*new, mm1);           /* U: mm1 = n7 n6 n5 n4 n3 n2 n1 n0 */
+            movq_r2r(mm0, mm2);            /* V: mm2 = r7 r6 r5 r4 r3 r2 r1 r0 */
 
-            /*
-             * ...but move the "motion flags" to memory before, in order to
-             * increment *smartmask_buffer properly below.
-             */
-            movq_r2m(mm0, mmtemp);           /* U */
-            pcmpeqb_r2r(mm4, mm3);           /* V: mm3 = 0xff where sX==0 */
+            /* These subtractions are saturated, i.e. won't go below 0. */
+            psubusb_r2r(mm1, mm0);         /* U: mm0 = (r7-n7) ... (r0-n0) */
+            psubusb_r2r(mm2, mm1);         /* V: mm1 = (n7-r7) ... (n0-r0) */
 
-            /* AND negates the target before anding. */
-            pandn_r2r(mm0, mm3);             /* U: mm3 = 0xff where dX>noise && sX>0 */
+            /* Each byte dX in mm0 is abs(nX-rX). */
+            por_r2r(mm1, mm0);             /* U: mm0 = d7 d6 d5 d4 d3 d2 d1 d0 */
 
-            movq_r2r(mm3, mm0);              /* U */
+            /* Expand the absolute differences to words in mm0 and mm1. */
+            movq_r2r(mm0, mm1);            /* U: mm1 = d7 d6 d5 d4 d3 d2 d1 d0 */
+            punpcklbw_r2r(mm4, mm0);       /* V: mm0 =    d3    d2    d1    d0 */
 
-            /* Add to *smartmask_buffer. This is probably the fastest way to do it. */
-            if (cnt->event_nr != cnt->prev_event) {
-                if (mmtemp.ub[0]) smartmask_buffer[0] += SMARTMASK_SENSITIVITY_INCR;
-                if (mmtemp.ub[1]) smartmask_buffer[1] += SMARTMASK_SENSITIVITY_INCR;
-                if (mmtemp.ub[2]) smartmask_buffer[2] += SMARTMASK_SENSITIVITY_INCR;
-                if (mmtemp.ub[3]) smartmask_buffer[3] += SMARTMASK_SENSITIVITY_INCR;
-                if (mmtemp.ub[4]) smartmask_buffer[4] += SMARTMASK_SENSITIVITY_INCR;
-                if (mmtemp.ub[5]) smartmask_buffer[5] += SMARTMASK_SENSITIVITY_INCR;
-                if (mmtemp.ub[6]) smartmask_buffer[6] += SMARTMASK_SENSITIVITY_INCR;
-                if (mmtemp.ub[7]) smartmask_buffer[7] += SMARTMASK_SENSITIVITY_INCR;
+            punpckhbw_r2r(mm4, mm1);       /* U: mm1 =    d7    d6    d5    d4 */
+
+            if (mask) {
+                /*
+                * Load and expand 8 mask bytes to words in mm2 and mm3. Then
+                * multiply by mm0 and mm1, respectively.
+                */
+                movq_m2r(*mask, mm2);      /* U: mm2 = m7 m6 m5 m4 m3 m2 m1 m0 */
+
+                movq_r2r(mm2, mm3);        /* U: mm3 = m7 m6 m5 m4 m3 m2 m1 m0 */
+                punpcklbw_r2r(mm4, mm2);   /* v: mm2 =    m3    m2    m1    m0 */
+
+                punpckhbw_r2r(mm4, mm3);   /* U: mm3 =    m7    m6    m5    m4 */
+                pmullw_r2r(mm2, mm0);      /* V: mm0 = (d3*m3) ... (d0*m0) */
+
+                pmullw_r2r(mm3, mm1);      /* U: mm1 = (d7*m7) ... (d4*m4) */
+
+                mask += 8;
+            } else {
+                /*
+                * Not using mask - multiply the absolute differences by 255. We
+                * do this by left-shifting 8 places and then subtracting dX.
+                */
+                movq_r2r(mm0, mm2);        /* U: mm2 =    d3    d2    d1    d0 */
+                psllw_i2r(8, mm0);         /* V: mm2 = (256*d3) ... (256*d0) */
+
+                movq_r2r(mm1, mm3);        /* U: mm3 =    d7    d6    d5    d4 */
+                psllw_i2r(8, mm1);         /* V: mm3 = (256*d7) ... (256*d4) */
+
+                psubusw_r2r(mm2, mm0);     /* U */
+                psubusw_r2r(mm3, mm1);     /* V */
             }
 
-            smartmask_buffer += 8;
-            smartmask_final += 8;
+            /*
+            * Next, compare the multiplied absolute differences with the multiplied
+            * noise level (repeated as 4 words in mm7), resulting in a "motion flag"
+            * for each pixel.
+            *
+            * Since pcmpgtw performs signed comparisons, we have to subtract noise,
+            * test for equality to 0 and then invert the result.
+            *
+            * Note that it is safe to generate the "motion flags" before the
+            * smartmask code, as all that can happen is that individual flags get
+            * reset to 0 because of the smartmask.
+            */
+            psubusw_r2r(mm7, mm0);         /* U: subtract by (multiplied) noise */
+            psubusw_r2r(mm7, mm1);         /* V */
+
+            pcmpeqw_r2r(mm4, mm0);         /* U: test for equality with 0 */
+            pcmpeqw_r2r(mm4, mm1);         /* V */
+
+            pand_r2r(mm6, mm0);            /* U: convert 0xffff -> 0x00ff */
+            pand_r2r(mm6, mm1);            /* V */
+
+            pxor_r2r(mm6, mm0);            /* U: invert the result */
+            pxor_r2r(mm6, mm1);            /* V */
+
+            /* Each fX is the "motion flag" = 0 for no motion, 0xff for motion. */
+            packuswb_r2r(mm1, mm0);        /* U: mm0 = f7 f6 f5 f4 f3 f2 f1 f0 */
+
+            if (smartmask_speed) {
+                /*
+                * Apply the smartmask. Basically, if *smartmask_final is 0, the
+                * corresponding "motion flag" in mm0 will be reset.
+                */
+                movq_m2r(*smartmask_final, mm3); /* U: mm3 = s7 s6 s5 s4 s3 s2 s1 s0 */
+
+                /*
+                * ...but move the "motion flags" to memory before, in order to
+                * increment *smartmask_buffer properly below.
+                */
+                movq_r2m(mm0, mmtemp);           /* U */
+                pcmpeqb_r2r(mm4, mm3);           /* V: mm3 = 0xff where sX==0 */
+
+                /* AND negates the target before anding. */
+                pandn_r2r(mm0, mm3);             /* U: mm3 = 0xff where dX>noise && sX>0 */
+
+                movq_r2r(mm3, mm0);              /* U */
+
+                /* Add to *smartmask_buffer. This is probably the fastest way to do it. */
+                /* TODO:  Revise this to use a loop */
+                if (cnt->event_nr != cnt->prev_event) {
+                    if (mmtemp.ub[0]) {
+                        smartmask_buffer[0] += SMARTMASK_SENSITIVITY_INCR;
+                    }
+                    if (mmtemp.ub[1]) {
+                        smartmask_buffer[1] += SMARTMASK_SENSITIVITY_INCR;
+                    }
+                    if (mmtemp.ub[2]) {
+                        smartmask_buffer[2] += SMARTMASK_SENSITIVITY_INCR;
+                    }
+                    if (mmtemp.ub[3]) {
+                        smartmask_buffer[3] += SMARTMASK_SENSITIVITY_INCR;
+                    }
+                    if (mmtemp.ub[4]) {
+                        smartmask_buffer[4] += SMARTMASK_SENSITIVITY_INCR;
+                    }
+                    if (mmtemp.ub[5]) {
+                        smartmask_buffer[5] += SMARTMASK_SENSITIVITY_INCR;
+                    }
+                    if (mmtemp.ub[6]) {
+                        smartmask_buffer[6] += SMARTMASK_SENSITIVITY_INCR;
+                    }
+                    if (mmtemp.ub[7]) {
+                        smartmask_buffer[7] += SMARTMASK_SENSITIVITY_INCR;
+                    }
+                }
+
+                smartmask_buffer += 8;
+                smartmask_final += 8;
+            }
+
+            movq_m2r(*new, mm2);           /* U: mm1 = n7 n6 n5 n4 n3 n2 n1 n0 */
+
+            /*
+            * Cancel out pixels in *new according to the "motion flags" in mm0.
+            * Each NX is either 0 or nX as from *new.
+            */
+            pand_r2r(mm0, mm2);            /* U: mm1 = N7 N6 N5 N4 N3 N2 N1 N0 */
+            psubb_r2r(mm0, mm4);           /* V: mm4 = 0x01 where dX>noise */
+
+            /*
+            * mm5 holds 8 separate counts - each one is increased according to
+            * the contents of mm4 (where each byte is either 0x00 or 0x01).
+            */
+            movq_r2m(mm2, *out);           /* U: this will stall */
+            paddusb_r2r(mm4, mm5);         /* V: add counts to mm5 */
+
+            /*
+            * Every 255th turn, we need to unload mm5 into the diffs variable,
+            * because otherwise the packed bytes will get saturated.
+            */
+            if (--unload == 0) {
+                /* Unload mm5 to memory and reset it. */
+                movq_r2m(mm5, mmtemp);     /* U */
+                pxor_r2r(mm5, mm5);        /* V: mm5 = 0 */
+
+                diffs += mmtemp.ub[0] + mmtemp.ub[1] + mmtemp.ub[2] + mmtemp.ub[3] +
+                        mmtemp.ub[4] + mmtemp.ub[5] + mmtemp.ub[6] + mmtemp.ub[7];
+                unload = 255;
+            }
+
+            out += 8;
+            ref += 8;
+            new += 8;
         }
 
-        movq_m2r(*new, mm2);           /* U: mm1 = n7 n6 n5 n4 n3 n2 n1 n0 */
-
         /*
-         * Cancel out pixels in *new according to the "motion flags" in mm0.
-         * Each NX is either 0 or nX as from *new.
-         */
-        pand_r2r(mm0, mm2);            /* U: mm1 = N7 N6 N5 N4 N3 N2 N1 N0 */
-        psubb_r2r(mm0, mm4);           /* V: mm4 = 0x01 where dX>noise */
-
-        /*
-         * mm5 holds 8 separate counts - each one is increased according to
-         * the contents of mm4 (where each byte is either 0x00 or 0x01).
-         */
-        movq_r2m(mm2, *out);           /* U: this will stall */
-        paddusb_r2r(mm4, mm5);         /* V: add counts to mm5 */
-
-        /*
-         * Every 255th turn, we need to unload mm5 into the diffs variable,
-         * because otherwise the packed bytes will get saturated.
-         */
-        if (--unload == 0) {
-            /* Unload mm5 to memory and reset it. */
-            movq_r2m(mm5, mmtemp);     /* U */
-            pxor_r2r(mm5, mm5);        /* V: mm5 = 0 */
-
+        * Check if there are diffs left in mm5 that need to be copied to the
+        * diffs variable.
+        */
+        if (unload < 255) {
+            movq_r2m(mm5, mmtemp);
             diffs += mmtemp.ub[0] + mmtemp.ub[1] + mmtemp.ub[2] + mmtemp.ub[3] +
-                     mmtemp.ub[4] + mmtemp.ub[5] + mmtemp.ub[6] + mmtemp.ub[7];
-            unload = 255;
+                    mmtemp.ub[4] + mmtemp.ub[5] + mmtemp.ub[6] + mmtemp.ub[7];
         }
 
-        out += 8;
-        ref += 8;
-        new += 8;
-    }
+        emms();
 
-    /*
-     * Check if there are diffs left in mm5 that need to be copied to the
-     * diffs variable.
-     */
-    if (unload < 255) {
-        movq_r2m(mm5, mmtemp);
-        diffs += mmtemp.ub[0] + mmtemp.ub[1] + mmtemp.ub[2] + mmtemp.ub[3] +
-                 mmtemp.ub[4] + mmtemp.ub[5] + mmtemp.ub[6] + mmtemp.ub[7];
-    }
-
-    emms();
-
-#endif
+    #endif
     /*
      * Note that the non-MMX code is present even if the MMX code is present.
      * This is necessary if the resolution is not a multiple of 8, in which
@@ -1168,8 +1234,9 @@ int alg_diff_standard(struct context *cnt, unsigned char *new)
     for (; i > 0; i--) {
         register unsigned char curdiff = (int)(abs(*ref - *new)); /* Using a temp variable is 12% faster. */
         /* Apply fixed mask */
-        if (mask)
+        if (mask) {
             curdiff = ((int)(curdiff * *mask++) / 255);
+        }
 
         if (smartmask_speed) {
             if (curdiff > noise) {
@@ -1180,11 +1247,13 @@ int alg_diff_standard(struct context *cnt, unsigned char *new)
                  * speed=10) we add 5 here. NOT related to the 5 at ratio-
                  * calculation.
                  */
-                if (cnt->event_nr != cnt->prev_event)
+                if (cnt->event_nr != cnt->prev_event) {
                     (*smartmask_buffer) += SMARTMASK_SENSITIVITY_INCR;
+                }
                 /* Apply smart_mask */
-                if (!*smartmask_final)
+                if (!*smartmask_final) {
                     curdiff = 0;
+                }
             }
             smartmask_final++;
             smartmask_buffer++;
@@ -1212,8 +1281,9 @@ static char alg_diff_fast(struct context *cnt, int max_n_changes, unsigned char
     int noise = cnt->noise;
     unsigned char *ref = imgs->ref;
 
-    if (!step % 2)
+    if (!step % 2) {
         step++;
+    }
     /* We're checking only 1 of several pixels. */
     max_n_changes /= step;
 
@@ -1223,8 +1293,9 @@ static char alg_diff_fast(struct context *cnt, int max_n_changes, unsigned char
         register unsigned char curdiff = (int)(abs((char)(*ref - *new))); /* Using a temp variable is 12% faster. */
         if (curdiff >  noise) {
             diffs++;
-            if (diffs > max_n_changes)
+            if (diffs > max_n_changes) {
                 return 1;
+            }
         }
         ref += step;
         new += step;
@@ -1242,8 +1313,9 @@ int alg_diff(struct context *cnt, unsigned char *new)
 {
     int diffs = 0;
 
-    if (alg_diff_fast(cnt, cnt->conf.threshold / 2, new))
+    if (alg_diff_fast(cnt, cnt->conf.threshold / 2, new)) {
         diffs = alg_diff_standard(cnt, new);
+    }
 
     return diffs;
 }
@@ -1258,14 +1330,17 @@ int alg_lightswitch(struct context *cnt, int diffs)
 {
     struct images *imgs = &cnt->imgs;
 
-    if (cnt->conf.lightswitch_percent < 0)
+    if (cnt->conf.lightswitch_percent < 0) {
         cnt->conf.lightswitch_percent = 0;
-    if (cnt->conf.lightswitch_percent > 100)
+    }
+    if (cnt->conf.lightswitch_percent > 100) {
         cnt->conf.lightswitch_percent = 100;
+    }
 
     /* Is lightswitch percent of the image changed? */
-    if (diffs > (imgs->motionsize * cnt->conf.lightswitch_percent / 100))
+    if (diffs > (imgs->motionsize * cnt->conf.lightswitch_percent / 100)) {
         return 1;
+    }
 
     return 0;
 }
@@ -1284,15 +1359,18 @@ int alg_switchfilter(struct context *cnt, int diffs, unsigned char *newimg)
     for (y = 0; y < cnt->imgs.height; y++) {
         line = 0;
         for (x = 0; x < cnt->imgs.width; x++) {
-            if (*(out++))
+            if (*(out++)) {
                 line++;
+            }
         }
 
-        if (line > cnt->imgs.width / 18)
+        if (line > cnt->imgs.width / 18) {
             vertlines++;
+        }
 
-        if (line > linediff * 2)
+        if (line > linediff * 2) {
             lines++;
+        }
     }
 
     if (vertlines > cnt->imgs.height / 10 && lines < vertlines / 3 &&
@@ -1332,8 +1410,10 @@ void alg_update_reference_frame(struct context *cnt, int action)
     unsigned char *smartmask = cnt->imgs.smartmask_final;
     unsigned char *out = cnt->imgs.img_motion.image_norm;
 
-    if (cnt->lastrate > 5)  /* Match rate limit */
+    /* Match rate limit */
+    if (cnt->lastrate > 5) {
         accept_timer /= (cnt->lastrate / 3);
+    }
 
     if (action == UPDATE_REF_FRAME) { /* Black&white only for better performance. */
         threshold_ref = cnt->noise * EXCLUDE_LEVEL_PERCENT / 100;
