@@ -22,8 +22,8 @@
     Copyright Jeroen Vreeken, 2002
 */
 
-#include "translate.h"
 #include "motion.h"
+#include "translate.h"
 #include "util.h"
 #include "logger.h"
 #include "picture.h"
@@ -86,19 +86,10 @@ static void exec_command(struct context *cnt, char *command, char *filename, int
     mystrftime(cnt, stamp, sizeof(stamp), command, &cnt->current_image->timestamp_tv, filename, filetype);
 
     if (!fork()) {
-        int i;
 
         /* Detach from parent */
         setsid();
 
-        /*
-         * Close any file descriptor except console because we will
-         * like to see error messages
-         */
-        for (i = getdtablesize() - 1; i > 2; i--) {
-            close(i);
-        }
-
         execl("/bin/sh", "sh", "-c", stamp, " &", NULL);
 
         /* if above function succeeds the program never reach here */
