@@ -620,7 +620,6 @@ void webu_process_action(struct webui_ctx *webui)
                     _("Quitting thread %d"),webui->thread_nbr);
                 webui->cntlst[indx]->restart = FALSE;
                 webui->cntlst[indx]->event_stop = TRUE;
-                webui->cntlst[indx]->event_user = TRUE;
                 webui->cntlst[indx]->finish = TRUE;
             }
         } else {
@@ -628,7 +627,6 @@ void webu_process_action(struct webui_ctx *webui)
                 _("Quitting thread %d"),webui->thread_nbr);
             webui->cnt->restart = FALSE;
             webui->cnt->event_stop = TRUE;
-            webui->cnt->event_user = TRUE;
             webui->cnt->finish = TRUE;
         }
 
@@ -638,7 +636,6 @@ void webu_process_action(struct webui_ctx *webui)
                 webui->cntlst[indx]->webcontrol_finish = TRUE;
                 webui->cntlst[indx]->restart = FALSE;
                 webui->cntlst[indx]->event_stop = TRUE;
-                webui->cntlst[indx]->event_user = TRUE;
                 webui->cntlst[indx]->finish = TRUE;
                 indx++;
             }
@@ -977,8 +974,18 @@ static int webu_failauth_check(struct webui_ctx *webui)
     }
 
     if (retcd) {
-        MOTION_LOG(ALR,TYPE_ALL, NO_ERRNO
+        if (mystrne(
+            _("Ignoring connection from: %s")
+            ,"Ignoring connection from: %s")) {
+            MOTION_LOG(ALR,TYPE_ALL, NO_ERRNO
             , _("Ignoring connection from: %s"), webui->clientip);
+        }
+        /* Do not translate the message below or change it
+         * in any way.  Other applications read the logs looking
+         * for this message so that the IP can be banned
+         */
+        MOTION_LOG(ALR,TYPE_ALL, NO_ERRNO
+            , "Ignoring connection from: %s", webui->clientip);
         SLEEP(2, 0);
     }
 
@@ -1690,30 +1697,15 @@ static void webu_mhd_deinit(void *cls, struct MHD_Connection *connection
     (void)toe;
 
     if (webui->cnct_type == WEBUI_CNCT_FULL ) {
-        pthread_mutex_lock(&webui->cnt->mutex_stream);
-            webui->cnt->stream_norm.cnct_count--;
-        pthread_mutex_unlock(&webui->cnt->mutex_stream);
-
+        webu_stream_deinit(webui, &webui->cnt->stream_norm);
     } else if (webui->cnct_type == WEBUI_CNCT_SUB ) {
-        pthread_mutex_lock(&webui->cnt->mutex_stream);
-            webui->cnt->stream_sub.cnct_count--;
-        pthread_mutex_unlock(&webui->cnt->mutex_stream);
-
+        webu_stream_deinit(webui, &webui->cnt->stream_sub);
     } else if (webui->cnct_type == WEBUI_CNCT_MOTION ) {
-        pthread_mutex_lock(&webui->cnt->mutex_stream);
-            webui->cnt->stream_motion.cnct_count--;
-        pthread_mutex_unlock(&webui->cnt->mutex_stream);
-
+        webu_stream_deinit(webui, &webui->cnt->stream_motion);
     } else if (webui->cnct_type == WEBUI_CNCT_SOURCE ) {
-        pthread_mutex_lock(&webui->cnt->mutex_stream);
-            webui->cnt->stream_source.cnct_count--;
-        pthread_mutex_unlock(&webui->cnt->mutex_stream);
-
+        webu_stream_deinit(webui, &webui->cnt->stream_source);
     } else if (webui->cnct_type == WEBUI_CNCT_STATIC ) {
-        pthread_mutex_lock(&webui->cnt->mutex_stream);
-            webui->cnt->stream_norm.cnct_count--;
-        pthread_mutex_unlock(&webui->cnt->mutex_stream);
-
+        webu_stream_deinit(webui, &webui->cnt->stream_norm);
     }
 
     webu_context_free(webui);
@@ -1859,7 +1851,7 @@ static char *webu_mhd_loadfile(const char *fname)
     if (fname == NULL) {
         file_char = NULL;
     } else {
-        infile = fopen(fname, "rb");
+        infile = myfopen(fname, "rbe");
         if (infile != NULL) {
             fseek(infile, 0, SEEK_END);
             file_size = ftell(infile);
